@@ -6,7 +6,7 @@ import typing
 
 import quicktions
 
-Real = typing.Union[float, fractions.Fraction, quicktions.Fraction, int]
+Real = float | fractions.Fraction | quicktions.Fraction | int
 """The main reason for this constant is a mypy issue with Pythons buildin
 [numbers module](https://docs.python.org/3/library/numbers.html) which
 is documented [here](https://github.com/python/mypy/issues/3186). Mypy
