@@ -62,11 +62,33 @@ class Event(abc.ABC):
 
     @functools.cached_property
     def _event_to_metrized_event(self):
-        # XXX: import in method to avoid circular import error
+        # Import in method to avoid circular import error
         return __import__(
             "mutwo.core_converters"
         ).core_converters.EventToMetrizedEvent()
 
+    @abc.abstractmethod
+    def _set_parameter(
+        self,
+        parameter_name: str,
+        object_or_function: typing.Callable[
+            [core_constants.ParameterType], core_constants.ParameterType
+        ]
+        | core_constants.ParameterType,
+        set_unassigned_parameter: bool,
+        id_set: set[int],
+    ) -> Event:
+        ...
+
+    @abc.abstractmethod
+    def _mutate_parameter(
+        self,
+        parameter_name: str,
+        function: typing.Callable[[core_constants.ParameterType], None] | typing.Any,
+        id_set: set[int],
+    ) -> Event:
+        ...
+
     # ###################################################################### #
     #                           public properties                            #
     # ###################################################################### #
@@ -160,9 +182,7 @@ class Event(abc.ABC):
     @abc.abstractmethod
     def get_parameter(
         self, parameter_name: str, flat: bool = False, filter_undefined: bool = False
-    ) -> typing.Union[
-        tuple[core_constants.ParameterType, ...], core_constants.ParameterType
-    ]:
+    ) -> tuple[core_constants.ParameterType, ...] | core_constants.ParameterType:
         """Return event attribute with the entered name.
 
         :param parameter_name: The name of the attribute that shall be returned.
@@ -193,17 +213,15 @@ class Event(abc.ABC):
         None
         """
 
-    @abc.abstractmethod
     def set_parameter(
         self,
         parameter_name: str,
-        object_or_function: typing.Union[
-            typing.Callable[
-                [core_constants.ParameterType], core_constants.ParameterType
-            ],
-            core_constants.ParameterType,
-        ],
+        object_or_function: typing.Callable[
+            [core_constants.ParameterType], core_constants.ParameterType
+        ]
+        | core_constants.ParameterType,
         set_unassigned_parameter: bool = True,
+        mutate: bool = True,
     ) -> typing.Optional[Event]:
         """Sets parameter to new value for all children events.
 
@@ -232,15 +250,33 @@ class Event(abc.ABC):
         >>> sequential_event.set_parameter('duration', lambda duration: duration * 2)
         >>> sequential_event.get_parameter('duration')
         (4, 6)
-        """
 
-    @abc.abstractmethod
+        **Warning:**
+
+        If there are multiple references of the same Event inside a
+        :class:`~mutwo.core_events.SequentialEvent` or a
+        ~mutwo.core_events.SimultaneousEvent`, ``set_parameter`` will
+        only be called once for each Event. So multiple references
+        of the same event will be ignored. This behaviour ensures,
+        that on a big scale level each item inside the
+        :class:`mutwo.core_events.abc.ComplexEvent` is treated equally
+        (so for instance the duration of each item is doubled, and
+        nor for some doubled and for those with references which
+        appear twice quadrupled).
+        """
+        return self._set_parameter(
+            parameter_name,
+            object_or_function,
+            set_unassigned_parameter=set_unassigned_parameter,
+            mutate=mutate,
+            id_set=set([]),
+        )
+
     def mutate_parameter(
         self,
         parameter_name: str,
-        function: typing.Union[
-            typing.Callable[[core_constants.ParameterType], None], typing.Any
-        ],
+        function: typing.Callable[[core_constants.ParameterType], None] | typing.Any,
+        mutate: bool = True,
     ) -> typing.Optional[Event]:
         """Mutate parameter with a function.
 
@@ -280,7 +316,26 @@ class Event(abc.ABC):
         >>> # now all pitches should be one octave higher (from 4 to 5)
         >>> sequential_event.get_parameter('pitch_list')
         ([WesternPitch(c5), WesternPitch(e5)],)
+
+        **Warning:**
+
+        If there are multiple references of the same Event inside a
+        :class:`~mutwo.core_events.SequentialEvent` or a
+        ~mutwo.core_events.SimultaneousEvent`, ``mutate_parameter`` will
+        only be called once for each Event. So multiple references
+        of the same event will be ignored. This behaviour ensures,
+        that on a big scale level each item inside the
+        :class:`mutwo.core_events.abc.ComplexEvent` is treated equally
+        (so for instance the duration of each item is doubled, and
+        nor for some doubled and for those with references which
+        appear twice quadrupled).
         """
+        return self._mutate_parameter(
+            parameter_name,
+            function,
+            mutate=mutate,
+            id_set=set([]),
+        )
 
     @core_utilities.add_copy_option
     def reset_tempo_envelope(self) -> Event:
@@ -410,7 +465,7 @@ class ComplexEvent(Event, abc.ABC, list[T], typing.Generic[T]):
     def __init_subclass__(
         cls, class_specific_side_attribute_tuple: tuple[str, ...] = tuple([])
     ):
-        # XXX: It's better to prove `class_specific_side_attribute_tuple`
+        # It's better to prove `class_specific_side_attribute_tuple`
         # as a class initialisation attribute instead of a simple class attribute,
         # because with a simple class attribute we have no guarantee that the
         # content of the parent class is available and we always have to explicitly
@@ -454,24 +509,61 @@ class ComplexEvent(Event, abc.ABC, list[T], typing.Generic[T]):
         return empty_copy
 
     @typing.overload
-    def __getitem__(self, index_or_slice: int) -> T:
+    def __getitem__(self, index_or_slice_or_tag: int) -> T:
         ...
 
     @typing.overload
-    def __getitem__(self, index_or_slice: slice) -> ComplexEvent[T]:
+    def __getitem__(self, index_or_slice_or_tag: slice) -> ComplexEvent[T]:
+        ...
+
+    @typing.overload
+    def __getitem__(self, index_or_slice_or_tag: str) -> T:
         ...
 
     def __getitem__(
-        self, index_or_slice: typing.Union[int, slice]
-    ) -> typing.Union[T, ComplexEvent[T]]:
-        event = super().__getitem__(index_or_slice)
-        if isinstance(index_or_slice, slice):
+        self, index_or_slice_or_tag: int | slice | str
+    ) -> T | ComplexEvent[T]:
+        try:
+            event = super().__getitem__(index_or_slice_or_tag)
+        except TypeError as error:
+            if isinstance(index_or_slice_or_tag, str):
+                return self.__getitem__(self._tag_to_index(index_or_slice_or_tag))
+            # It can't be a tag, therefore simply raise
+            # original exception.
+            raise error
+        if isinstance(index_or_slice_or_tag, slice):
             empty_event = self.empty_copy()
             empty_event.extend(event)
             return empty_event
         else:
             return event
 
+    @typing.overload
+    def __setitem__(self, index_or_slice_or_tag: int, event: core_events.abc.Event):
+        ...
+
+    @typing.overload
+    def __setitem__(self, index_or_slice_or_tag: slice, event: core_events.abc.Event):
+        ...
+
+    @typing.overload
+    def __setitem__(self, index_or_slice_or_tag: str, event: core_events.abc.Event):
+        ...
+
+    def __setitem__(
+        self, index_or_slice_or_tag: int | slice | str, event: core_events.abc.Event
+    ):
+        try:
+            super().__setitem__(index_or_slice_or_tag, event)
+        except TypeError as error:
+            if isinstance(index_or_slice_or_tag, str):
+                return self.__setitem__(
+                    self._tag_to_index(index_or_slice_or_tag), event
+                )
+            # It can't be a tag, therefore simply raise
+            # original exception.
+            raise error
+
     def __eq__(self, other: typing.Any) -> bool:
         """Test for checking if two objects are equal."""
         try:
@@ -509,6 +601,25 @@ class ComplexEvent(Event, abc.ABC, list[T], typing.Generic[T]):
     #                           private methods                              #
     # ###################################################################### #
 
+    # Keep private because:
+    #   (1) Then we can later change the internal implementation of
+    #       ComplexEvent (for instance: no longer inheriting from list).
+    #   (2) It's not sure if tag_to_index is valuable for end users of
+    #       ComplexEvent
+    def _tag_to_index(self, tag: str) -> int:
+        # Find index of an event by its tag.
+        # param tag: The `tag` of the event which shall be found.
+        # type tag: str
+
+        for event_index, event in enumerate(self):
+            try:
+                event_tag = event.tag
+            except AttributeError:
+                continue
+            if tag == event_tag:
+                return event_index
+        raise KeyError(f"No event found with tag = '{tag}'.")
+
     def _assert_start_in_range(self, start: core_parameters.abc.Duration):
         """Helper method to make sure that start < event.duration.
 
@@ -517,6 +628,47 @@ class ComplexEvent(Event, abc.ABC, list[T], typing.Generic[T]):
         if self.duration < start:
             raise core_utilities.InvalidStartValueError(start, self.duration)
 
+    def _apply_once_per_event(
+        self, method_name: str, *args, id_set: set[int], **kwargs
+    ):
+        for event in self:
+            if (event_id := id(event)) not in id_set:
+                id_set.add(event_id)
+                getattr(event, method_name)(*args, id_set=id_set, **kwargs)
+
+    @core_utilities.add_copy_option
+    def _set_parameter(  # type: ignore
+        self,
+        parameter_name: str,
+        object_or_function: typing.Callable[
+            [core_constants.ParameterType], core_constants.ParameterType
+        ]
+        | core_constants.ParameterType,
+        set_unassigned_parameter: bool,
+        id_set: set[int],
+    ) -> ComplexEvent[T]:
+        self._apply_once_per_event(
+            "_set_parameter",
+            parameter_name,
+            object_or_function,
+            id_set=id_set,
+            set_unassigned_parameter=set_unassigned_parameter,
+        )
+
+    @core_utilities.add_copy_option
+    def _mutate_parameter(  # type: ignore
+        self,
+        parameter_name: str,
+        function: typing.Callable[[core_constants.ParameterType], None] | typing.Any,
+        id_set: set[int],
+    ) -> ComplexEvent[T]:
+        self._apply_once_per_event(
+            "_mutate_parameter",
+            parameter_name,
+            function,
+            id_set=id_set,
+        )
+
     # ###################################################################### #
     #                           public methods                               #
     # ###################################################################### #
@@ -597,7 +749,7 @@ class ComplexEvent(Event, abc.ABC, list[T], typing.Generic[T]):
             if flat:
                 parameter_value_list.extend(parameter_value_tuple)
             else:
-                # XXX: Simple events should be added without tuple, they only
+                # Simple events should be added without tuple, they only
                 # provide one parameter.
                 if is_simple_event:
                     if parameter_value_tuple:
@@ -606,37 +758,6 @@ class ComplexEvent(Event, abc.ABC, list[T], typing.Generic[T]):
                     parameter_value_list.append(parameter_value_tuple)
         return tuple(parameter_value_list)
 
-    @core_utilities.add_copy_option
-    def set_parameter(  # type: ignore
-        self,
-        parameter_name: str,
-        object_or_function: typing.Union[
-            typing.Callable[
-                [core_constants.ParameterType], core_constants.ParameterType
-            ],
-            core_constants.ParameterType,
-        ],
-        set_unassigned_parameter: bool = True,
-    ) -> ComplexEvent[T]:
-        [
-            event.set_parameter(
-                parameter_name,
-                object_or_function,
-                set_unassigned_parameter=set_unassigned_parameter,
-            )
-            for event in self
-        ]
-
-    @core_utilities.add_copy_option
-    def mutate_parameter(  # type: ignore
-        self,
-        parameter_name: str,
-        function: typing.Union[
-            typing.Callable[[core_constants.ParameterType], None], typing.Any
-        ],
-    ) -> ComplexEvent[T]:
-        [event.mutate_parameter(parameter_name, function) for event in self]
-
     @core_utilities.add_copy_option
     def remove_by(  # type: ignore
         self, condition: typing.Callable[[Event], bool]
@@ -757,7 +878,7 @@ class ComplexEvent(Event, abc.ABC, list[T], typing.Generic[T]):
     def squash_in(
         self, start: core_parameters.abc.Duration, event_to_squash_in: Event
     ) -> typing.Optional[ComplexEvent[T]]:
-        """Time-based insert of a new event into the present event.
+        """Time-based insert of a new event with overriding given event.
 
         :param start: Absolute time where the event shall be inserted.
         :param event_to_squash_in: the event that shall be squashed into
@@ -766,7 +887,9 @@ class ComplexEvent(Event, abc.ABC, list[T], typing.Generic[T]):
             If set to ``True`` the object itself will be changed and the function will
             return the changed object. Default to ``True``.
 
-        Squash in a new event to the present event.
+        Unlike `ComplexEvent.slide_in` the events duration won't change.
+        If there is already an event at `start` this event will be shortened
+        or removed.
 
         **Example:**
 
@@ -777,6 +900,33 @@ class ComplexEvent(Event, abc.ABC, list[T], typing.Generic[T]):
         SequentialEvent([SimpleEvent(duration = 1), SimpleEvent(duration = 1.5), SimpleEvent(duration = 0.5)])
         """
 
+    @abc.abstractmethod
+    def slide_in(
+        self, start: core_parameters.abc.Duration, event_to_slide_in: Event
+    ) -> ComplexEvent[T]:
+        """Time-based insert of a new event into the present event.
+
+        :param start: Absolute time where the event shall be inserted.
+        :param event_to_slide_in: the event that shall be slide into
+            the present event.
+        :param mutate: If ``False`` the function will return a copy of the given object.
+            If set to ``True`` the object itself will be changed and the function will
+            return the changed object. Default to ``True``.
+
+        Unlike `ComplexEvent.squash_in` the events duration will be prolonged
+        by the event which is added. If there is an event at `start` the
+        event will be split into two parts, but it won't be shortened or
+        processed in any other way.
+
+        **Example:**
+
+        >>> from mutwo import core_events
+        >>> sequential_event = core_events.SequentialEvent([core_events.SimpleEvent(3)])
+        >>> sequential_event.slide_in(1, core_events.SimpleEvent(1.5))
+        >>> print(sequential_event)
+        SequentialEvent([SimpleEvent(duration = 1), SimpleEvent(duration = 1.5), SimpleEvent(duration = 2)])
+        """
+
     @abc.abstractmethod
     def split_child_at(
         self, absolute_time: core_parameters.abc.Duration
@@ -796,3 +946,47 @@ class ComplexEvent(Event, abc.ABC, list[T], typing.Generic[T]):
         >>> sequential_event
         SequentialEvent([SimpleEvent(duration = 1), SimpleEvent(duration = 2)])
         """
+
+    @abc.abstractmethod
+    def extend_until(
+        self,
+        duration: core_parameters.abc.Duration,
+        duration_to_white_space: typing.Optional[
+            typing.Callable[[core_parameters.abc.Duration], Event]
+        ] = None,
+        prolong_simple_event: bool = True,
+    ) -> ComplexEvent:
+        """Prolong event until at least `duration` by appending an empty event.
+
+        :param duration: Until which duration the event shall be extended.
+            If event is already longer than or equal to given `duration`,
+            nothing will be changed. For :class:`~mutwo.core_events.SimultaneousEvent`
+            the default value is `None` which is equal to the duration of
+            the `SimultaneousEvent`.
+        :type duration: core_parameters.abc.Duration
+        :param duration_to_white_space: A function which creates the 'rest' or
+            'white space' event from :class:`~mutwo.core_parameters.abc.Duration`.
+            If this is ``None`` `mutwo` will fall back to use the default function
+            which is `mutwo.core_events.configurations.DEFAULT_DURATION_TO_WHITE_SPACE`.
+            Default to `None`.
+        :type duration_to_white_space: typing.Optional[typing.Callable[[core_parameters.abc.Duration], Event]]
+        :param prolong_simple_event: If set to ``True`` `mutwo` will prolong a single
+            :class:`~mutwo.core_events.SimpleEvent` inside a :class:`~mutwo.core_events.SimultaneousEvent`.
+            If set to ``False`` `mutwo` will raise an :class:`~mutwo.core_utilities.ImpossibleToExtendUntilError`
+            in case it finds a single `SimpleEvent` inside a `SimultaneousEvent`.
+            This doesn't effect `SimpleEvent` inside a `SequentialEvent`, here we can
+            simply append a new white space event.
+        :type prolong_simple_event: bool
+        :param mutate: If ``False`` the function will return a copy of the given object.
+            If set to ``True`` the object itself will be changed and the function will
+            return the changed object. Default to ``True``.
+        :type mutate: bool
+
+        **Example:**
+
+        >>> from mutwo import core_events
+        >>> s = core_events.SequentialEvent([core_events.SimpleEvent(1)])
+        >>> s.extend_until(10)
+        >>> print(s)
+        SequentialEvent([SimpleEvent(duration = DirectDuration(duration = 1)), SimpleEvent(duration = DirectDuration(duration = 9))])
+        """
