@@ -81,6 +81,40 @@ class SimpleEvent(core_events.abc.Event):
         )
         return "{}({})".format(type(self).__name__, ", ".join(attribute_iterator))
 
+    # ###################################################################### #
+    #                           private methods                              #
+    # ###################################################################### #
+
+    @core_utilities.add_copy_option
+    def _set_parameter(
+        self,
+        parameter_name: str,
+        object_or_function: typing.Callable[
+            [core_constants.ParameterType], core_constants.ParameterType
+        ]
+        | core_constants.ParameterType,
+        set_unassigned_parameter: bool,
+        id_set: set[int],
+    ) -> SimpleEvent:
+        old_parameter = self.get_parameter(parameter_name)
+        if set_unassigned_parameter or old_parameter is not None:
+            if hasattr(object_or_function, "__call__"):
+                new_parameter = object_or_function(old_parameter)
+            else:
+                new_parameter = object_or_function
+            setattr(self, parameter_name, new_parameter)
+
+    @core_utilities.add_copy_option
+    def _mutate_parameter(
+        self,
+        parameter_name: str,
+        function: typing.Callable[[core_constants.ParameterType], None] | typing.Any,
+        id_set: set[int],
+    ) -> SimpleEvent:
+        parameter = self.get_parameter(parameter_name)
+        if parameter is not None:
+            function(parameter)
+
     # ###################################################################### #
     #                           properties                                   #
     # ###################################################################### #
@@ -88,7 +122,7 @@ class SimpleEvent(core_events.abc.Event):
     @property
     def _parameter_to_print_tuple(self) -> tuple[str, ...]:
         """Return tuple of attribute names which shall be printed for repr."""
-        # XXX: Fix infinite circular loop (due to 'tempo_envelope')
+        # Fix infinite circular loop (due to 'tempo_envelope')
         # and avoid printing too verbose parameters.
         return tuple(
             filter(
@@ -108,7 +142,7 @@ class SimpleEvent(core_events.abc.Event):
         return tuple(
             attribute
             for attribute in dir(self)
-            # XXX: We have to use 'and' (lazy evaluation) instead of
+            # We have to use 'and' (lazy evaluation) instead of
             #      'all', to avoid redundant checks!
             #
             # no private attributes
@@ -139,17 +173,11 @@ class SimpleEvent(core_events.abc.Event):
     ) -> core_constants.ParameterType:
         return getattr(self, parameter_name, None)
 
-    @core_utilities.add_copy_option
+    # Update docstring
     def set_parameter(  # type: ignore
         self,
-        parameter_name: str,
-        object_or_function: typing.Union[
-            typing.Callable[
-                [core_constants.ParameterType], core_constants.ParameterType
-            ],
-            core_constants.ParameterType,
-        ],
-        set_unassigned_parameter: bool = True,
+        *args,
+        **kwargs,
     ) -> SimpleEvent:
         """Sets event parameter to new value.
 
@@ -191,25 +219,7 @@ class SimpleEvent(core_events.abc.Event):
         10
         """
 
-        old_parameter = self.get_parameter(parameter_name)
-        if set_unassigned_parameter or old_parameter is not None:
-            if hasattr(object_or_function, "__call__"):
-                new_parameter = object_or_function(old_parameter)
-            else:
-                new_parameter = object_or_function
-            setattr(self, parameter_name, new_parameter)
-
-    @core_utilities.add_copy_option
-    def mutate_parameter(  # type: ignore
-        self,
-        parameter_name: str,
-        function: typing.Union[
-            typing.Callable[[core_constants.ParameterType], None], typing.Any
-        ],
-    ) -> SimpleEvent:
-        parameter = self.get_parameter(parameter_name)
-        if parameter is not None:
-            function(parameter)
+        return super().set_parameter(*args, **kwargs)
 
     def metrize(self, mutate: bool = True) -> SimpleEvent:
         metrized_event = self._event_to_metrized_event(self)
@@ -298,7 +308,7 @@ class SequentialEvent(core_events.abc.ComplexEvent, typing.Generic[T]):
     #                        private  methods                                #
     # ###################################################################### #
 
-    # XXX: We need to have a private "_cut_off" method to simplify
+    # We need to have a private "_cut_off" method to simplify
     # overriding the public "cut_off" method in children classes
     # of SequentialEvent. This is necessary, because the implementation
     # of "squash_in" makes use of "_cut_off". In this way it is possible
@@ -382,7 +392,18 @@ class SequentialEvent(core_events.abc.ComplexEvent, typing.Generic[T]):
 
         duration_iterator = (event.duration.duration_in_floats for event in self)
         absolute_time_tuple = tuple(
-            core_utilities.accumulate_from_n(duration_iterator, 0)
+            # We need to round each duration again after accumulation,
+            # because floats were summed which could lead to
+            # potential floating point errors again, which will
+            # lead to bad errors later (for instance in
+            # core_utilities.scale).
+            map(
+                lambda d: core_utilities.round_floats(
+                    d,
+                    core_parameters.configurations.ROUND_DURATION_TO_N_DIGITS,
+                ),
+                core_utilities.accumulate_from_n(duration_iterator, 0),
+            )
         )
         return absolute_time_tuple[:-1], absolute_time_tuple[-1]
 
@@ -430,13 +451,13 @@ class SequentialEvent(core_events.abc.ComplexEvent, typing.Generic[T]):
     # ###################################################################### #
 
     def get_event_index_at(
-        self, absolute_time: typing.Union[core_parameters.abc.Duration, typing.Any]
+        self, absolute_time: core_parameters.abc.Duration | typing.Any
     ) -> typing.Optional[int]:
         """Get index of event which is active at the passed absolute_time.
 
         :param absolute_time: The absolute time where the method shall search
             for the active event.
-        :type absolute_time: typing.Union[core_parameters.abc.Duration, typing.Any]
+        :type absolute_time: core_parameters.abc.Duration | typing.Any
         :return: Index of event if there is any event at the requested absolute time
             and ``None`` if there isn't any event.
 
@@ -468,13 +489,13 @@ class SequentialEvent(core_events.abc.ComplexEvent, typing.Generic[T]):
         )
 
     def get_event_at(
-        self, absolute_time: typing.Union[core_parameters.abc.Duration, typing.Any]
+        self, absolute_time: core_parameters.abc.Duration | typing.Any
     ) -> typing.Optional[T]:
         """Get event which is active at the passed absolute_time.
 
         :param absolute_time: The absolute time where the method shall search
             for the active event.
-        :type absolute_time: typing.Union[core_parameters.abc.Duration, typing.Any]
+        :type absolute_time: core_parameters.abc.Duration | typing.Any
         :return: Event if there is any event at the requested absolute time
             and ``None`` if there isn't any event.
 
@@ -533,7 +554,7 @@ class SequentialEvent(core_events.abc.ComplexEvent, typing.Generic[T]):
             if cut_out_start < cut_out_end:
                 event.cut_out(cut_out_start, cut_out_end)
             elif not (
-                # XXX: Support special case of events with duration = 0.
+                # Support special case of events with duration = 0.
                 event.duration == 0
                 and event_start >= start
                 and event_start <= end
@@ -562,7 +583,7 @@ class SequentialEvent(core_events.abc.ComplexEvent, typing.Generic[T]):
     @core_utilities.add_copy_option
     def squash_in(  # type: ignore
         self,
-        start: typing.Union[core_parameters.abc.Duration, typing.Any],
+        start: core_parameters.abc.Duration | typing.Any,
         event_to_squash_in: core_events.abc.Event,
     ) -> SequentialEvent[T]:
         start = core_events.configurations.UNKNOWN_OBJECT_TO_DURATION(start)
@@ -619,9 +640,22 @@ class SequentialEvent(core_events.abc.ComplexEvent, typing.Generic[T]):
 
             self.insert(insert_index, event_to_squash_in)
 
+    @core_utilities.add_copy_option
+    def slide_in(
+        self,
+        start: core_parameters.abc.Duration,
+        event_to_slide_in: core_events.abc.Event,
+    ) -> SequentialEvent[T]:
+        start = core_events.configurations.UNKNOWN_OBJECT_TO_DURATION(start)
+        start_in_floats = start.duration_in_floats
+        self._assert_start_in_range(start_in_floats)
+        self[:], b = self.split_at(start)
+        self.extend([event_to_slide_in] + b)
+        return self
+
     @core_utilities.add_copy_option
     def split_child_at(
-        self, absolute_time: typing.Union[core_parameters.abc.Duration, typing.Any]
+        self, absolute_time: core_parameters.abc.Duration | typing.Any
     ) -> SequentialEvent[T]:
         absolute_time_in_floats = core_events.configurations.UNKNOWN_OBJECT_TO_DURATION(
             absolute_time
@@ -652,10 +686,64 @@ class SequentialEvent(core_events.abc.ComplexEvent, typing.Generic[T]):
             self[event_index] = first_event
             self.insert(event_index, second_event)
 
+    def split_at(
+        self, absolute_time: core_parameters.abc.Duration
+    ) -> tuple[SequentialEvent, SequentialEvent]:
+        absolute_time = core_events.configurations.UNKNOWN_OBJECT_TO_DURATION(
+            absolute_time
+        )
+        absolute_time_tuple = self.absolute_time_tuple
+        # Only run expensive 'split_at' in case we can't simply
+        # split via slice.
+        try:
+            index = absolute_time_tuple.index(absolute_time)
+        except ValueError:
+            return super().split_at(absolute_time)
+        else:
+            return self[:index].copy(), self[index:].copy()
+
+    @core_utilities.add_copy_option
+    def extend_until(
+        self,
+        duration: core_parameters.abc.Duration,
+        duration_to_white_space: typing.Optional[
+            typing.Callable[[core_parameters.abc.Duration], core_events.abc.Event]
+        ] = None,
+        prolong_simple_event: bool = True,
+    ) -> SequentialEvent[T]:
+        duration = core_events.configurations.UNKNOWN_OBJECT_TO_DURATION(duration)
+        duration_to_white_space = (
+            duration_to_white_space
+            or core_events.configurations.DEFAULT_DURATION_TO_WHITE_SPACE
+        )
+        if (difference := duration - self.duration) > 0:
+            self.append(duration_to_white_space(difference))
+
 
 class SimultaneousEvent(core_events.abc.ComplexEvent, typing.Generic[T]):
     """Event-Object which contains other Event-Objects which happen at the same time."""
 
+    # ###################################################################### #
+    #                       private static methods                           #
+    # ###################################################################### #
+
+    @staticmethod
+    def _extend_ancestor(
+        ancestor: core_events.abc.Event, event: core_events.abc.Event
+    ):
+        match ancestor:
+            case core_events.SequentialEvent():
+                ancestor.extend(event)
+            case core_events.SimultaneousEvent():
+                try:
+                    ancestor.concatenate_by_tag(event)
+                except core_utilities.NoTagError:
+                    ancestor.concatenate_by_index(event)
+            # We can't concatenate to a simple event.
+            # We also can't concatenate to anything else.
+            case _:
+                raise core_utilities.ConcatenationError(ancestor, event)
+
     # ###################################################################### #
     #                           properties                                   #
     # ###################################################################### #
@@ -675,8 +763,8 @@ class SimultaneousEvent(core_events.abc.ComplexEvent, typing.Generic[T]):
     @core_utilities.add_copy_option
     def cut_out(  # type: ignore
         self,
-        start: typing.Union[core_parameters.abc.Duration, typing.Any],
-        end: typing.Union[core_parameters.abc.Duration, typing.Any],
+        start: core_parameters.abc.Duration | typing.Any,
+        end: core_parameters.abc.Duration | typing.Any,
     ) -> SimultaneousEvent[T]:
         start, end = (
             core_events.configurations.UNKNOWN_OBJECT_TO_DURATION(unknown_object)
@@ -701,7 +789,7 @@ class SimultaneousEvent(core_events.abc.ComplexEvent, typing.Generic[T]):
     @core_utilities.add_copy_option
     def squash_in(  # type: ignore
         self,
-        start: typing.Union[core_parameters.abc.Duration, typing.Any],
+        start: core_parameters.abc.Duration | typing.Any,
         event_to_squash_in: core_events.abc.Event,
     ) -> SimultaneousEvent[T]:
         start = core_events.configurations.UNKNOWN_OBJECT_TO_DURATION(start)
@@ -714,6 +802,21 @@ class SimultaneousEvent(core_events.abc.ComplexEvent, typing.Generic[T]):
             except AttributeError:
                 raise core_utilities.ImpossibleToSquashInError(self, event_to_squash_in)
 
+    @core_utilities.add_copy_option
+    def slide_in(
+        self,
+        start: core_parameters.abc.Duration,
+        event_to_slide_in: core_events.abc.Event,
+    ) -> SimultaneousEvent[T]:
+        start = core_events.configurations.UNKNOWN_OBJECT_TO_DURATION(start)
+        self._assert_start_in_range(start)
+        for event in self:
+            try:
+                event.slide_in(start, event_to_slide_in)  # type: ignore
+            # Simple events don't have a 'slide_in' method.
+            except AttributeError:
+                raise core_utilities.ImpossibleToSlideInError(self, event_to_slide_in)
+
     @core_utilities.add_copy_option
     def split_child_at(
         self, absolute_time: core_constants.DurationType
@@ -726,6 +829,111 @@ class SimultaneousEvent(core_events.abc.ComplexEvent, typing.Generic[T]):
                 split_event = event.split_at(absolute_time)
                 self[event_index] = SequentialEvent(split_event)
 
+    @core_utilities.add_copy_option
+    def extend_until(
+        self,
+        duration: typing.Optional[core_parameters.abc.Duration] = None,
+        duration_to_white_space: typing.Optional[
+            typing.Callable[[core_parameters.abc.Duration], core_events.abc.Event]
+        ] = None,
+        prolong_simple_event: bool = True,
+    ) -> SequentialEvent:
+        duration = (
+            self.duration
+            if duration is None
+            else core_events.configurations.UNKNOWN_OBJECT_TO_DURATION(duration)
+        )
+        duration_to_white_space = (
+            duration_to_white_space
+            or core_events.configurations.DEFAULT_DURATION_TO_WHITE_SPACE
+        )
+        for event in self:
+            try:
+                event.extend_until(
+                    duration, duration_to_white_space, prolong_simple_event
+                )
+            # SimpleEvent
+            except AttributeError:
+                if prolong_simple_event:
+                    if (difference := duration - event.duration) > 0:
+                        event.duration += difference
+                else:
+                    raise core_utilities.ImpossibleToExtendUntilError(event)
+
+    @core_utilities.add_copy_option
+    def concatenate_by_index(self, other: SimultaneousEvent) -> SimultaneousEvent:
+        """Concatenate with other :class:`~mutwo.core_events.SimultaneousEvent` along their indices.
+
+        :param other: The other `SimultaneousEvent` with which to concatenate.
+            The other `SimultaneousEvent` can contain more or less events.
+        :type other: SimultaneousEvent
+        :param mutate: If ``False`` the function will return a copy of the given object.
+            If set to ``True`` the object itself will be changed and the function will
+            return the changed object. Default to ``True``.
+        :type mutate: bool
+        :raises core_utilities.ConcatenationError: If there are any :class:`SimpleEvent`
+            inside a :class:`SimultaneousEvent`.
+
+        **Example:**
+
+        >>> from mutwo import core_events
+        >>> s = core_events.SimultaneousEvent(
+        >>>     [core_events.SequentialEvent([core_events.SimpleEvent(1)])]
+        >>> )
+        >>> s.concatenate_by_index(s)
+        SimultaneousEvent([SequentialEvent([SimpleEvent(duration = DirectDuration(duration = 1)), SimpleEvent(duration = DirectDuration(duration = 1))])])
+        """
+        self_duration = self.duration
+        self.extend_until(self_duration)
+        for index, event in enumerate(other.copy()):
+            try:
+                ancestor = self[index]
+            except IndexError:
+                event.slide_in(0, core_events.SimpleEvent(self_duration))
+                self.append(event)
+            else:
+                self._extend_ancestor(ancestor, event)
+
+    @core_utilities.add_copy_option
+    def concatenate_by_tag(self, other: SimultaneousEvent) -> SimultaneousEvent:
+        """Concatenate with other :class:`~mutwo.core_events.SimultaneousEvent` along their tags.
+
+        :param other: The other `SimultaneousEvent` with which to concatenate.
+            The other `SimultaneousEvent` can contain more or less events.
+        :type other: SimultaneousEvent
+        :param mutate: If ``False`` the function will return a copy of the given object.
+            If set to ``True`` the object itself will be changed and the function will
+            return the changed object. Default to ``True``.
+        :type mutate: bool
+        :return: Concatenated event.
+        :raises core_utilities.NoTagError: If any child event doesn't have a 'tag'
+            attribute.
+        :raises core_utilities.ConcatenationError: If there are any :class:`SimpleEvent`
+            inside a :class:`SimultaneousEvent`.
+
+        **Example:**
+
+        >>> from mutwo import core_events
+        >>> s = core_events.SimultaneousEvent(
+        >>>     [core_events.TaggedSequentialEvent([core_events.SimpleEvent(1)], tag="test")]
+        >>> )
+        >>> s.concatenate_by_tag(s)
+        SimultaneousEvent([TaggedSequentialEvent([SimpleEvent(duration = DirectDuration(duration = 1)), SimpleEvent(duration = DirectDuration(duration = 1))])])
+        """
+        self_duration = self.duration
+        self.extend_until(self_duration)
+        for tagged_event in other.copy():
+            if not hasattr(tagged_event, "tag"):
+                raise core_utilities.NoTagError(tagged_event)
+            tag = tagged_event.tag
+            try:
+                ancestor = self[tag]
+            except KeyError:
+                tagged_event.slide_in(0, core_events.SimpleEvent(self_duration))
+                self.append(tagged_event)
+            else:
+                self._extend_ancestor(ancestor, tagged_event)
+
 
 @core_utilities.add_tag_to_class
 class TaggedSimpleEvent(SimpleEvent):
