@@ -9,13 +9,18 @@ __all__ = (
     "InvalidAverageValueStartAndEndWarning",
     "InvalidStartValueError",
     "InvalidPointError",
+    "ImpossibleToPutInError",
     "ImpossibleToSquashInError",
+    "ImpossibleToSlideInError",
+    "ImpossibleToExtendUntilError",
     "InvalidStartAndEndValueError",
     "InvalidCutOutStartAndEndValuesError",
     "SplitUnavailableChildError",
     "NoSolutionFoundError",
     "EmptyEnvelopeError",
     "UndefinedReferenceWarning",
+    "ConcatenationError",
+    "NoTagError",
 )
 
 
@@ -58,17 +63,38 @@ class InvalidPointError(Exception):
         )
 
 
-class ImpossibleToSquashInError(TypeError):
-    def __init__(self, event_to_be_squashed_into, event_to_squash_in):
+class ImpossibleToPutInError(TypeError):
+    def __init__(self, event_to_be_put_into, event_to_put_in, method_name):
+        m = method_name
         super().__init__(
-            f"Can't squash '{event_to_squash_in}' in '{event_to_be_squashed_into}'. "
+            f"Can't {m} '{event_to_put_in}' in '{event_to_be_put_into}'. "
             "Does the SimultaneousEvent contain SimpleEvents or events that inherit"
-            " from SimpleEvent? For being able to squash in, the"
+            f" from SimpleEvent? For being able to {m} in, the"
             " SimultaneousEvent needs to only contain SequentialEvents or"
             " SimultaneousEvents."
         )
 
 
+class ImpossibleToSquashInError(ImpossibleToPutInError):
+    def __init__(self, event_to_be_squashed_into, event_to_squash_in):
+        super().__init__(event_to_be_squashed_into, event_to_squash_in, "squash")
+
+
+class ImpossibleToSlideInError(TypeError):
+    def __init__(self, event_to_be_slided_into, event_to_slide_in):
+        super().__init__(event_to_be_slided_into, event_to_slide_in, "slide")
+
+
+class ImpossibleToExtendUntilError(TypeError):
+    def __init__(self, event_to_extend_until):
+        super().__init__(
+            f"Can't extend '{event_to_extend_until}' of type"
+            f"'{type(event_to_extend_until)}' which resides inside a "
+            "SimultaneousEvent. Set 'prolong_simple_event' to 'True' in"
+            "case you want simple events to be prolonged."
+        )
+
+
 class InvalidStartAndEndValueError(Exception):
     def __init__(self, start, end):
         super().__init__(
@@ -112,3 +138,26 @@ class UndefinedReferenceWarning(RuntimeWarning):
             "doesn't know attribute 'reference'."
             " Therefore reference has been set to 1."
         )
+
+
+class ConcatenationError(TypeError):
+    def __init__(self, ancestor, event):
+        super().__init__(
+            f"Can't concatenate event '{event}' to event '{ancestor}' "
+            f"of type '{type(ancestor)}'. It is only possible to"
+            " concatenate a new event to events which are instances of "
+            "SequentialEvent or SimultaneousEvent. To fix this bug you can"
+            f" put your event '{ancestor}' inside a SequentialEvent or"
+            " a SimultaneousEvent."
+        )
+
+
+class NoTagError(Exception):
+    def __init__(self, event_without_tag):
+        super().__init__(
+            "It's not possible to concatenate an event "
+            "with the 'concatenate_by_tag' method if not "
+            "all child events have tags. Here 'mutwo' detected the "
+            f"child event '{str(event_without_tag)[:50]}...' "
+            "which doesn't have any 'tag' attribute."
+        )
